/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Defines vrpsx_embedding_main_t; see its comments.
 */

#ifndef VRPSX_EMBEDDING_MAIN_H
#define VRPSX_EMBEDDING_MAIN_H 1

#include "vrpsx_embedding_initial.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* The type of a main-like function that uses vrpsx_embedding_t.
 *
 * The caller promises that (initial != NULL) and that the members of
 * (*initial) are not NULL unless and until the called function modifies them.
 *
 * This function type and its comments constitute an identified interface:
 * the clients are the function definitions and the implementations are
 * the callers.  A library may list such an implementation by using the
 * interface defined by "vrpsx_identified_function.h".
 *
 * This function interface extends vrpsx_system_main_t, meaning that any
 * caller that satisfies this interface can call a function that expects
 * to be called according to the vrpsx_system_main_t interface.  However,
 * that might not always be the highest performance approach because the
 * preparation of the outer environment may require time and space that
 * would be wasted on a function that is unaware of its existence.  Even
 * though some functions might query (*initial) and discover additional
 * data, we do not expect that to be the typical pattern of usage when
 * the interface defined by "vrpsx_identified_function.h" is employed,
 * because the library could just declare two entry points, one for
 * this interface and one for vrpsx_system_main_t.
 *
 * Note that vrpsx_embedding_initial_t extends vrpsx_system_initial_t at
 * offset 0.  A 0 offset is necessary in order for this function interface
 * to be an extension of vrpsx_system_main_t.  (We might be able to expand
 * the function interface extension relationship in order to handle nonzero
 * offsets, but the code could get quite complex and our guess is that any
 * benefits would be minimal, especially since, as noted above, we might
 * prefer to ignore even the extension relationship that we currently have.)
 *
 * Extends: vrpsx_system_main_t.
 */
typedef int vrpsx_embedding_main_t(
    vrpsx_embedding_initial_t *initial);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_EMBEDDING_MAIN_H */
